#ifndef _KERNEL_COMMUNICATE_H
#define _KERNEL_COMMUNICATE_H

#include <stddef.h>

#define MAX_PAYLOAD		1024	/*消息最大负载为1024字节*/
/* #define NETLINK_KYSEC_KMOD	27 */

#define NETLINK_KYSEC_GROUP_HASH 2
#define NETLINK_KYSEC_HASH	27
#define KYSEC_NETLINK_MESG_MAX	1000

/* kysec special label type */
enum {
	EXECTL_VERIFIED_TYPE = 1,
	EXECTL_ORIGINAL_TYPE,
	EXECTL_KYSOFT_TYPE,
	EXECTL_TRUSTED_TYPE,
	EXECTL_UNKNOWN_TYPE,
	EXECTL_PARENT_TYPE,
	EXECTL_SCRIPT_TYPE
};

enum kysec_message_type {
	NETCTL_INIT = 0,
	NETCTL_ADD,
	NETCTL_DEL,
	KYSEC_NOTIFY,
	LABEL_INIT,
	LABEL_ADD,
	LABEL_DEL,
	KMOD_ADD,
	KMOD_DEL,
	PPRO_ADD,
	PPRO_DEL,
	PPRO_CRITICAL_PID,
	PPRO_PID,
	DEVCTL_ADD,
	DEVCTL_DEL,
	SSID_ADD,
	SSID_DEL,
	DEVCTL_ADD_SM,
	DEVCTL_DEL_SM,
	PBLK_ADD,
	PBLK_DEL
};

enum kysec_notify_type {
	NOTIFY_LABEL = 0,
	NOTIFY_NETCTL,
	NOTIFY_FPRO,
	NOTIFY_IPT,
	NOTIFY_DEVCTL
};

static const char * const kysec_label_types_str[] = {
	NULL,
	"verified",
	"original",
	"kysoft",
	"trusted",
	"unknown",
	"parent",
	"script",
};

#ifdef __cplusplus
extern "C" {
#endif

/*
 * 功能：内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @kmod_name: 表示内核模块名
 */
int kysec_kmod_communicate_with_kernel(int mesg_type, char *kmod_name);

/*
 * 功能：(涉密场景)内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @kmod_name: 表示内核模块名
 */
int kysec_kmod_communicate_with_kernel_sm(int mesg_type, char *kmod_name);

/*
 * 功能：讲exectl标记转换为内核标记类型
 * exectl_label：exectl标记
 */
int kysec_convert_exectl_label_to_label_type(char *exectl_label);

/*
 * 功能：内核通讯接口
 * 
 */
int kysec_hash_communicate_with_kernel(int mesg_type, char *hash, int label_type);

/*
 * 功能：应用网络管控内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @hash: 从xml文件中读取出来的hash值
 * @netctl_type: 应用网络管控类型
 */
int kysec_netctl_communicate_with_kernel(int mesg_type, char *hash, int netctl_type);

/*
 * 功能：进程保护内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @hash: 从xml文件中读取出来的hash值
 * @ppro_type: 进程保护类型
 */
int kysec_ppro_communicate_with_kernel(int mesg_type, char *hash, int ppro_type);

/*
 * 功能：(涉密场景)进程保护内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @hash: 从xml文件中读取出来的hash值
 * @ppro_type: 进程保护类型
 */
int kysec_ppro_communicate_with_kernel_sm(int mesg_type, char *hash, int ppro_type);

/*
 * 功能：程序黑名单内核通讯接口
 * @mesg_type: 取值enum kysec_message_type，表示消息的操作类型
 * @hash: 从xml文件中读取出来的hash值
 */
int kysec_pblk_communicate_with_kernel(int mesg_type, char *hash);

/**
 * @name:  kysec_devctl_communicate_with_kernel
 * @Descripttion: 设备管控与内核通讯接口
 * @msg:
 * @param {int} mesg_type   消息的操作类型
 * @param {void} *data      数据
 * @param {int} data_len   消息长度
 * @return {*}
 */
int kysec_devctl_communicate_with_kernel(int mesg_type, void *data, int data_len);

#ifdef __cplusplus
}
#endif

#endif
