#ifndef _kYSEC_PPRO_H
#define _kYSEC_PPRO_H

#include <limits.h>


#define XML_HASH_SIZE	64


enum ppro_type {
	PPRO_SUBS = 1,	/* can't be killed */
	PPRO_KEY	/* key process, also can't be killed, */
};

struct kysec_ppro_kernel {
	char path[PATH_MAX];
	char hash[XML_HASH_SIZE + 1];
	int type;
};

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 获取文件路径的hash值
 */
char *kysec_ppro_get_path_hash(const char *filepath);

/**
 * 添加一个程序信息节点到xml文件
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 * 返回值: 0：成功，1：包含该path的节点已经存在，2：该path在黑名单中，-1：出错
 */
int kysec_ppro_add(char *path, int type);

/** 
 * 从xml文件中删除一个节点
 * @path: 程序路径
 * 返回值: 0:成功，-1:出错
 */
int kysec_ppro_remove(char *path);

/** 
 * 更新xml文件的某一个节点
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 *
 * 返回值: 0:成功，1:xml文件中不存在路径为@path的节点，-1:失败
 */
int kysec_ppro_update(char *path, int type);

/**
 * 获取指定路径path在xml文件中的节点数据
 */
struct kysec_ppro_kernel *kysec_ppro_read_node_with_path(char *filepath);

/**
 * 获取xml文件中所有节点数据
 * 返回值: kysec_ppro_kernel指针数组，大小为@count
 */
struct kysec_ppro_kernel *kysec_ppro_read_all_data(int *count);

/**
 * 获取 ppro.conf 中 action 的值，调用后需释放 action
 */
int kysec_ppro_get_action(char **action);

/**
 * 涉密场景：添加一个程序信息节点到xml文件
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 * 返回值: 0：成功，1：包含该path的节点已经存在，2：该path在黑名单中，-1：出错
 */
int kysec_ppro_add_sm(char *path, int type);

/** 
 * 涉密场景：从xml文件中删除一个节点
 * @path: 程序路径
 * 返回值: 0:成功，-1:出错
 */
int kysec_ppro_remove_sm(char *path);

/** 
 * 涉密场景：更新xml文件的某一个节点
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 *
 * 返回值: 0:成功，1:xml文件中不存在路径为@path的节点，-1:失败
 */
int kysec_ppro_update_sm(char *path, int type);

/**
 * 涉密场景：获取指定路径path在xml文件中的节点数据
 */
struct kysec_ppro_kernel *kysec_ppro_read_node_with_path_sm(char *filepath);

/**
 * 涉密场景：获取xml文件中所有节点数据
 * 返回值: kysec_ppro_kernel指针数组，大小为@count
 */
struct kysec_ppro_kernel *kysec_ppro_read_all_data_sm(int *count);

/** 
 * 涉密场景：检查文件是否存在在涉密场景
 * @hash: 程序路径hash值
 *
 * 返回值: 0:存在，-1:不存在
 */
int kysec_ppro_check_in_sm(char *hash);

#ifdef __cplusplus
}
#endif

#endif // _kYSEC_PPRO_H
